package net.scheinerman.grapher;

import java.awt.Color;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.RenderingHints;
import java.util.ArrayList;

import javax.swing.JPanel;

import net.scheinerman.phoenix.interpreter.exceptions.PhoenixRuntimeException;
import net.scheinerman.phoenix.interpreter.variables.NumberVariable;
import net.scheinerman.phoenix.runner.InteractivePhoenixEnvironment;

public class GraphPanel extends JPanel {

	private double xMax = 10;
	private double xMin = -10;
	private double xScale = 1;
	private double yMax = 10;
	private double yMin = -10;
	private double yScale = 1;
	
	private static final Color BACKGROUND = Color.WHITE;
	private static final Color AXES = Color.GRAY;
	
	private static final Color[] FUNC_COLORS = {
		Color.RED,
		Color.BLUE,
		Color.GREEN,
		Color.CYAN,
		Color.PINK,
		Color.ORANGE,
		Color.MAGENTA,
		Color.YELLOW,
		Color.GRAY
	};

	private static final long serialVersionUID = 1L;
	
	private ArrayList<GraphFunction> functions = new ArrayList<GraphFunction>();

	private InteractivePhoenixEnvironment phxEnv = new InteractivePhoenixEnvironment(false);
	
	public GraphPanel(ArrayList<GraphFunction> functions) {
		this.functions = functions;
		phxEnv.addModule(new GrapherPhoenixModule());
		phxEnv.setOutputStream(null);
		phxEnv.runCode("global num x = 0");
		phxEnv.runCode("global num y = 0");
	}
	
	public void paint(Graphics g) {
		g.clearRect(0, 0, getWidth(), getHeight());
		g.setColor(BACKGROUND);
		g.fillRect(0, 0, getWidth(), getHeight());
		
		drawAxes(g);
		int index = 0;
		for(GraphFunction f : functions) {
			g.setColor(FUNC_COLORS[index % FUNC_COLORS.length]);
			f.setError(!drawGraph(g, f));
			index++;
		}
	}
	
	public void drawAxes(Graphics g) {
		g.setColor(AXES);
		if(xMin <= 0 && xMax >= 0) {
			int x0 = xToPixel(0);
			g.drawLine(x0, 0, x0, getHeight());
			double yTickStart = yMin - yMin % yScale;
			double yTickEnd = yMax + yMax % yScale;
			for(double tick = yTickStart; tick <= yTickEnd; tick++) {
				int tickPixel = yToPixel(tick);
				g.drawLine(x0 - 5, tickPixel, x0 + 5, tickPixel);
			}
		}
		if(yMin <= 0 && yMax >= 0) {
			int y0 = yToPixel(0);
			g.drawLine(0, y0, getWidth(), y0);
			double xTickStart = xMin - xMin % xScale;
			double xTickEnd = xMax + xMax % xScale;
			for(double tick = xTickStart; tick <= xTickEnd; tick++) {
				int tickPixel = xToPixel(tick);
				g.drawLine(tickPixel, y0 - 5, tickPixel, y0 + 5);
			}
		}
	}
	
	public boolean drawGraph(Graphics g, GraphFunction function) {
		String code = function.getFunctionCode();
		if(code == null || code.trim().isEmpty()) return false;
		Graphics2D g2d = (Graphics2D)g;
		g2d.setRenderingHint(RenderingHints.KEY_ANTIALIASING,
                			 RenderingHints.VALUE_ANTIALIAS_ON);
		double lastX = Double.NaN;
		double lastY = Double.NaN;
		for(int i = -1; i < getWidth() + 1; i+= 2) {
			double x = pixelToX(i);
			double y = 0;
			phxEnv.runCode("x = " + x);
			try {
				phxEnv.runCode("y = " + code + "");
				NumberVariable phxVarY = (NumberVariable)phxEnv.getVAT().getLast().get("y");
				y = phxVarY.value();
			} catch(PhoenixRuntimeException e) {
				e.printErrorMessage();
				return false;
			}
			
			boolean lastInBounds = lastX >= xMin && lastX <= xMax && lastY >= yMin && lastY <= yMax;
			boolean inBounds = x >= xMin && x <= xMax && y >= yMin && y <= yMax;
			
			
			if(Double.isNaN(lastX) || Double.isNaN(lastY)) {
				if(inBounds) {
					g2d.drawLine(xToPixel(x), yToPixel(y), xToPixel(x), yToPixel(y));
				}
			} else if(lastInBounds || inBounds) {
				g2d.drawLine(xToPixel(lastX), yToPixel(lastY), xToPixel(x), yToPixel(y));
			}
			lastX = x;
			lastY = y;
		}
		return true;
	}

	public int xToPixel(double x) {		
		return (int)((x - xMin) / (xMax - xMin) * getWidth());
	}
	
	public int yToPixel(double y) {		
		return getHeight() - (int)((y - yMin) / (yMax - yMin) * getHeight());
	}
	
	public double pixelToX(int pixel) {
		return ((double)pixel / getWidth()) * (xMax - xMin) + xMin;
	}
	
	public double pixelToY(int pixel) {
		pixel = getHeight() - pixel;
		return ((double)pixel / getHeight()) * (yMax - yMin) + yMin;
	}
}
